package com.jprofiler.api.agent.probe;

import com.jprofiler.agent.probe.structures.MetaDataImpl;

/**
 * The meta data defines the capabilities of a probe. At startup, each probe is queried once for its meta data via a call to
 * {@link Probe#getMetaData()}. In that method, you create an instance with
 * {@link #create(String)} and call its methods to configure the probe. The meta data provides a fluent interface, so
 * you can append method calls like in <tt>ProbeMetaData.create("Test").events(true).recordOnStartup(false)</tt>.
 * <p>
 *     The <b>help topic on custom probes</b> in the bundled documentation explains how to develop custom probes.
 * </p>
 */
public abstract class ProbeMetaData {

    /**
     * Creates an meta-data instance. For custom probes defined the JProfiler GUI, you should not call this method, since a
     * meta data instance is already passed to the meta-data script. In that case, the name and description of the probe are
     * already set.
     * @param name the name of the probe. This name is displayed in the JProfiler GUI.
     * @return the meta-data instance
     */
    public static ProbeMetaData create(String name) {
        return new MetaDataImpl(name);
    }

    /**
     * Sets the description of the probe. The description is displayed in the probes view and the custom probe configuration
     * or the session settings. If this method is not called, a default description will be displayed.
     * @param description the description
     * @return this instance
     */
    public abstract ProbeMetaData description(String description);

    /**
     * Configures if the probe collects payload information. The default value is <tt>false</tt>.
     * If set to <tt>true</tt>, the hot spots view will be shown. The {@link InterceptorContext#createPayloadInfo(String)}
     * and {@link InterceptorContext#createPayloadInfo(String, Object)} and published via
     * {@link InterceptorContext#addPayloadInfo(PayloadInfo)} for standalone payloads or via <tt>create</tt> methods
     * in the {@link InterceptorContext} that take a {@link PayloadInfo} argument.
     * <p>
     *     This only makes sense for {@link InterceptorProbe interceptor probes}.
     * </p>
     * @param payload the value
     * @return this instance
     */
    public abstract ProbeMetaData payload(boolean payload);

    /**
     * Configures if probe retain single events. The default value is <tt>false</tt>. If set to <tt>true</tt>,
     * the events view will be shown. If the event count exceeds 100000, events will be consolidated. Events are created
     * with the <tt>create</tt> methods in {@link ProbeContext} and {@link InterceptorContext} and are published with
     * {@link ProbeContext#addEvent(ProbeEvent)}.
     * @param events the value
     * @return this instance
     */
    public abstract ProbeMetaData events(boolean events);

    /**
     * Configures if the probe collects telemetry data. The default value is <tt>false</tt>. If set to <tt>true</tt>,
     * the telemetries view will be shown. Telemetries must then be configured with calls to
     * {@link #addCustomTelemetry(String, Unit, float)} and {@link #addOpenControlObjectTelemetry(String)}.
     * Telemetry data is sampled calls to {@link TelemetryProbe#fillTelemetryData(ProbeContext, int[])}.
     * <p>
     *     This only makes sense for {@link TelemetryProbe telemetry probes}.
     * </p>
     * @param telemetry the value
     * @return this instance
     */
    public abstract ProbeMetaData telemetry(boolean telemetry);

    /**
     * Configures if the probe publishes control objects. The default value is <tt>false</tt>. If set to <tt>true</tt>,
     * the control objects view will be shown. Control objects are created with calls to
     * {@link ProbeContext#createOpenEvent(String, Object)} or {@link InterceptorContext#createOpenEvent(PayloadInfo, Object)}.
     * @param controlObjectView the value
     * @return this instance
     */
    public abstract ProbeMetaData controlObjectView(boolean controlObjectView);

    /**
     * Configures if the probe should record data on startup. The default value is <tt>true</tt>.
     * In any case, the user can start and stop recording the JProfiler GUI.
     * @param startRecording the value
     * @return this instance
     */
    public abstract ProbeMetaData recordOnStartup(boolean startRecording);

    /**
     * Configures if the probe should annotate data into the call tree view. The default value is <tt>false</tt>.
     * If set to <tt>true</tt>, you should also call the {@link #payloadName(String)} method, so you can distinguish
     * payload from different probes in the call tree.
     * @param annotatePayloads the value
     * @return this instance
     */
    public abstract ProbeMetaData annotatePayloads(boolean annotatePayloads);

    /**
     * Sets the name of the payload. Then the probe annotates data in the the call tree, the container node for the
     * payload of this probe shows this name.
     * <p>
     *     This only has an effect if {@link #annotatePayloads(boolean)} is called with
     *     the argument <tt>true</tt>.
     * </p>
     * @param payloadName the name
     * @return this instance
     */
    public abstract ProbeMetaData payloadName(String payloadName);


    /**
     * Starts a group of telemetries that can be displayed in the same graph. The telemetries have to have the same units
     * and their values should be in the sme order of magnitude. All subsequent calls to
     * {@link #addCustomTelemetry(String, Unit, float)} will be added to this group. To start a new group, you have to call
     * this method again. If you never call this method, all telemetries will be in separate graphs.
     * <p>
     *   This only makes sense if the {@link #telemetry(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the telemetry group
     * @return this instance
     */
    public abstract ProbeMetaData startTelemetryGroup(String name);

    /**
     * Configures a telemetry that shows the number of open control objects. This only makes sense if your probe supports
     * control objects and has called {@link #controlObjectView(boolean)} with the argument <tt>true</tt>.
     * <p>
     *   This only makes sense if the {@link #telemetry(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the telemetry
     * @return this instance
     */
    public abstract ProbeMetaData addOpenControlObjectTelemetry(String name);

    /**
     * Configures a custom telemetry. Telemetry values are sampled by calls to
     * {@link TelemetryProbe#fillTelemetryData(ProbeContext, int[])}. The order of the invocations of this configuration
     * method define the indices in the data array of the sampling method.
     * <p>
     *   This only makes sense if the {@link #telemetry(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the telemetry
     * @param unit the unit of the measurement
     * @param factor the number by which the measurements should be divided to get the numeric value for the vertical axis.
     *               If your measurements directly correspond to the selected unit, pass <tt>1f</tt>.
     * @return this instance
     */
    public abstract ProbeMetaData addCustomTelemetry(String name, Unit unit, float factor);

    /**
     * Configures an additional data column for probe events. Additional data is set by a call to {@link ProbeEvent#additionalData(Object[])}.
     * The order of the invocations of this configuration method define the indices in the data array of the
     * <tt>additionalData</tt> method.
     * <p>
     *     This only makes sense if the {@link #events(boolean)} method has been called with
     *     the argument <tt>true</tt>.
     * </p>
     * @param name the name of the additional data column. This is displayed as the column header in the events view.
     * @param dataType the data type of the additional data column
     * @return this instance
     */
    public abstract ProbeMetaData addAdditionalData(String name, DataType dataType);

    /**
     * Configures an additional data column for control objects. Additional data is set when control objects are opened with
     * a call to {@link ProbeContext#createOpenEvent(String, Object, Object[])} or {@link InterceptorContext#createOpenEvent(String, Object, Object[])}.
     * The order of the invocations of this configuration method define the indices in the data array of the
     * <tt>createOpenEvent</tt> methods.
     * <p>
     *   This only makes sense if the {@link #controlObjectView(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the additional data column. This is displayed as the column header in the control objects view.
     * @param dataType the data type of the additional data column
     * @param nested If the column should be displayed in a nested table. This makes sense if the data is more for debugging purposes,
     * contains very long text values or if you have a lot of additional data columns. For example, the built-in process
     * probe shows several columns in a nested table.
     * @return this instance
     */
    public abstract ProbeMetaData addAdditionalControlObjectData(String name, DataType dataType, boolean nested);

    /**
     * Configures the custom event types and their names. The index in the array becomes the ID of the custom event type
     * and is used in calls to {@link ProbeContext#createCustomEvent(String, int, Object)} and
     * {@link InterceptorContext#startCustomState(Object, int)}. Custom events that are associated with control objects
     * are also shown in the time line view in the way the horizontal bar of a control object is colored along the time axis.
     * To configure the colors of custom events, call {@link #customColors(int[])}.
     * @param typeNames the names of the custom events. Not that the index in the array becomes the ID of the custom event.
     * @return this instance
     */
    public abstract ProbeMetaData customTypeNames(String[] typeNames);

    /**
     * Configures the colors of the custom events in the time line view.
     * @param customColors the colors as RGB values. The RGB value is the result of the <tt>getRGB()</tt> method on
     * a <tt>java.awt.Color</tt> object. The array index of each color corresponds to the custom event index in the call to
     * {@link #customTypeNames(String[])}.
     * @return this instance
     * @see #customTypeNames(String[])
     */
    public abstract ProbeMetaData customColors(int[] customColors);

    /**
     * Sets the name of the automatic event type for the opening of a control object. This name is displayed in the
     * event type selector of control object and events view as well as in the event type column of the
     * events view. For example, the file probe sets this value to "File opened".
     * <p>
     *   This only makes sense if the {@link #controlObjectView(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the event type
     * @return this instance
     * @see #closeTypeName(String)
     */
    public abstract ProbeMetaData openTypeName(String name);

    /**
     * Sets the name of the automatic event type for the closing of a control object. This name is displayed in the
     * event type selector of control object and events view as well as in the event type column of the
     * events view. For example, the file probe sets this value to "File closed".
     * <p>
     *   This only makes sense if the {@link #controlObjectView(boolean)} method has been
     *   called with the argument <tt>true</tt>.
     * </p>
     * @param name the name of the event type
     * @return this instance
     * @see #openTypeName(String) (String)
     */
    public abstract ProbeMetaData closeTypeName(String name);

}
